%% 
%%% Determine the thresholds for different pulse numbers by fitting
%%% the squared crater diameter R_C^2 vs. ln(F) for each pulse number
%%% 
%%% INPUT:
%%% A file RCD.csv , a comma-delimited file with
%%%                  element (1,1) = empty
%%%                          row 1 = pulse number
%%%                       column 1 = fluence in J/cm
%%%    starting from element (2,2) = crater DIAMETERS in m
%%%
%%%  Data sets with less than 3 experimental values [diameter vs. ln(F)]
%%%  are automatically kicked out
%%%  
%%% OUTPUT:
%%% A graph for every pulse number N and a file FvsN.csv, containing
%%% pulse number in the first column and damage threshold in J/cm in
%%% the second column. This file can be used as input for the 
%%% fit of the incubation parameters by the script IncubationCurveFit.m

clc
clear

%% Read experimental data from file

datacsv = 'RCD';
filename=[datacsv,'.csv'];
FF=csvread(filename);                % read data from file
[Fmax,Nmax] = size(FF);              % # of columns and rows
D = FF(2:Fmax,2:Nmax);               % strip fluences & pulse numbers
FX = FF(2:Fmax,1); F0 = FX' ;        % vector with fluences
N = FF(1,2:Nmax); ExN = N   ;        % vector with pulse numbers
Fmax = Fmax - 1 ; Nmax = Nmax - 1;   % # of fluences, # of pulse numbers
w = zeros(1,Nmax);                   % vector for fitted beam radii
Fth = zeros(1,Nmax);                 % vector for fitted threshold fluences

for i0=(1:Nmax);                     % loop for all pulse numbers
ExD1 = D(:,i0) ; ExD = ExD1';  % diameters vs fluence for this pulse number
ExRsq = zeros(1,Fmax);
ExRsq = (ExD./2).^2;           % get squared radii from diameters for fit

%% remove missing experimental values that are read as zero by csvread
Ftemp=F0;
i1=1;                            % FOR loop does not work, since ...
while i1 <= numel(ExRsq)         %    numel(ExRsq) changes in loop
        if ExRsq(i1) == 0                
           ExRsq(i1) = [];  % kick zero value out, shorten the vector by 1
           Ftemp(i1) = [];  % kick fluence of this pair out, shorten vector
        else
            i1 = i1 + 1;   % only advance when nothing is kicked out, ...
        end                %  otherwise it advances automatically by ...
end                        %  Nachrutschen :-)

%% Fit
if numel(Ftemp) > 2         % remove sets that have < 3 data points
[xData, yData] = prepareCurveData( Ftemp, ExRsq );

ft = fittype('LiuFun(w,Fth,x)');

opts = fitoptions( ft );
opts.Display = 'final';
opts.Lower = [0.01 5];
opts.MaxIter = 100000;
opts.Robust = 'off';
opts.StartPoint = [1 100];
opts.Upper = [2 2000];
opts.TolFun = 1e-12;
opts.TolX = 1e-12;

[fitresult, gof] = fit( xData, yData, ft, opts );

inz = coeffvalues(fitresult);
Fth(i0) = inz(1);
w(i0) = inz(2);
%% calculate RMS of fit
RMS=0;
for i3 = (1:numel(Ftemp))
       RMS = RMS + (ExRsq(i3)  - LiuFun(w(i0),Fth(i0),Ftemp(i3)))^2;
end
RMS = sqrt(RMS/numel(Ftemp));
%% prepare fitted function for plotting
RC = zeros(1,numel(Ftemp));
for i5 = (1:numel(Ftemp));
   RC(i5) = LiuFun(w(i0),Fth(i0),Ftemp(i5));
end

%% plot fitted function and experimental data
figure;
semilogx(Ftemp,RC,'LineWidth',2);      % plot fitted function
xlabel('fluence [ J/cm ]');
ylabel('squared crater radius [ m ]');
hold all;
plot(Ftemp,ExRsq,'--rs','LineStyle','none',...   % plot experimental values
    'Marker','D',...
    'MarkerEdgeColor','k',...
    'MarkerFaceColor','r',...
    'MarkerSize',10);
sttr = {['Fth =  ',num2str(Fth(i0),'%5.3f'),' J/cm'], ...
        ['1/e-radius = ',num2str(w(i0),'%5.1f'),' m'], ...
        ['RMS =  ',num2str(RMS,'%2.3f'),' m']};
annotation('textbox',[.6 .25 .1 .1],'String',sttr)

end % end of IF loop removing sets with < 3 data points

end % FOR loop pulse number

%% remove the vector elements that were not fitted due to < 3 data points
while i1 <= numel(Fth)    % FOR loop does not work, since ...
        if Fth(i1) == 0                % numel(ExRsq) changes in loop
           Fth(i1) = [];  % kick zero value out
           ExN(i1) = [];  % kick corresponding pulse# ...
           w(i1) = [];    %         and radius out & shorten vector by 1
        else
            i1 = i1 + 1;   % only advance when nothing is kicked out
        end                % otherwise it advances automatically
end                        % durch Nachrutschen

datafilename=['FvsN.csv'] ;
MAT = cat(3,ExN',Fth',w') ;
csvwrite(datafilename,MAT) ;


